﻿using SushiBarContracts.BindingModels;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;
using SushiBarDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace SushiBarDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int SushiId { get; set; }
        [Required]
        public int Count { get; set; }
        [Required]
        public double Sum { get; set; }
        [Required]
        public OrderStatus Status { get; set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; set; } = DateTime.Now;
        public DateTime? DateImplement { get; set; }
        [Required]
        Sushi sushi { get; set; } = new();

        public static Order Create(SushiBarDatabase context, OrderBindingModel model)
        {
            return new Order()
            {
                Id = model.Id,
                SushiId = model.SushiId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                sushi = context.Sushis.First(x => x.Id == model.Id)
            };
        }
        public void Update(OrderBindingModel model)
        {
            Id = model.Id;
            SushiId = model.SushiId;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SushiId = SushiId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            SushiName = sushi.SushiName
        };
        public void UpdateSushi(SushiBarDatabase context, OrderBindingModel model)
        {
            sushi = context.Sushis.First(x => x.Id == model.Id);
        }
    }
}
